#ifndef AS1115_H
#define AS1115_H

#include "mbed.h"

typedef struct
{
    uint8_t device_number;
    uint8_t decode_mode;
    uint8_t intensity;
    uint8_t scan_limit;
}max7219_configuration_t;

class AS1115
{
    public:
        const static uint8_t DP = 0x40;
        
        typedef enum
        {
            AS1115_BCD_0=0x00,
            AS1115_BCD_1,
            AS1115_BCD_2,
            AS1115_BCD_3,
            AS1115_BCD_4,
            AS1115_BCD_5,
            AS1115_BCD_6,
            AS1115_BCD_7,
            AS1115_BCD_8,
            AS1115_BCD_9,
            AS1115_BCD_DASH,            
            AS1115_BCD_E,
            AS1115_BCD_H,
            AS1115_BCD_L,
            AS1115_BCD_P,
            AS1115_BCD_BLANK=0xF
        }bcd_font;
        
        typedef enum
        {
            AS1115_HEX_0=0x00,
            AS1115_HEX_1,
            AS1115_HEX_2,
            AS1115_HEX_3,
            AS1115_HEX_4,
            AS1115_HEX_5,
            AS1115_HEX_6,
            AS1115_HEX_7,
            AS1115_HEX_8,
            AS1115_HEX_9,
            AS1115_HEX_A,
            AS1115_HEX_B,
            AS1115_HEX_C,
            AS1115_HEX_D,
            AS1115_HEX_E,
            AS1115_HEX_F
        }hex_font;
            
        typedef enum
        {
            // DIGIT REGISTERS
            AS1115_D0 = 0x01,
            AS1115_D1,
            AS1115_D2,
            AS1115_D3,
            AS1115_D4,
            AS1115_D5,
            AS1115_D6,
            AS1115_D7,
            
            // CONTROL REGISTERS
            AS1115_DECODE_MODE,
            AS1115_GLOBAL_INTENSITY,
            AS1115_SCAN_LIMIT,
            AS1115_SHUTDOWN = 0x0C,
            AS1115_SELF_ADDRESSING = 0x2D, // ok fine lets be awkward
            AS1115_FEATURE =0x0E,
            AS1115_DISPLAY_TEST_MODE,
            AS1115_DIG0_DIG1_INTENSITY,
            AS1115_DIG2_DIG3_INTENSITY,
            AS1115_DIG4_DIG5_INTENSITY,
            AS1115_DIG6_DIG7_INTENSITY,
            
            // DIAGNOSTIC REGISTERS
            AS1115_DIAGNOSTIC_DIGIT_0,
            AS1115_DIAGNOSTIC_DIGIT_1,
            AS1115_DIAGNOSTIC_DIGIT_2,
            AS1115_DIAGNOSTIC_DIGIT_3,
            AS1115_DIAGNOSTIC_DIGIT_4,
            AS1115_DIAGNOSTIC_DIGIT_5,
            AS1115_DIAGNOSTIC_DIGIT_6,
            AS1115_DIAGNOSTIC_DIGIT_7,
            AS1115_KEYA,
            AS1115_KEYB=0x1D
        }registers;
            
        
        AS1115(I2C &display, const uint8_t address);
        AS1115(PinName sda, PinName sck, const uint8_t address);
        
        void init();
        
        // shutdown mode options
        void enable(bool hard_reset=false);
        void disable(bool hard_reset=false);
        
        // decode mode options
        void enable_decode();
        void disable_decode();
        
        // intensity
        void set_global_intensity(uint8_t level);
        
        // scan limit
        void set_num_digits(uint8_t num_digits);
        
        
        // display test mode
        void start_display_test();
        void stop_display_test();
        
        void write(uint8_t reg, uint8_t data);
        
        void write_digit(uint8_t digit_num, uint8_t val);
        
        void set_self_addresses();
        void set_font(bool hex_font);
      
    private:
    
        I2C &_display;
        uint8_t _address;
};
#endif /* AS1115_H*/